<?php
declare(strict_types=1);

namespace CakeProducts\Controller;

use Cake\Log\Log;
use CakeProducts\Controller\AppController;
use CakeProducts\Service\InternalCatalogManagerService;

/**
 * ProductCategories Controller
 *
 * @property \CakeProducts\Model\Table\ProductCategoriesTable $ProductCategories
 */
class ProductCategoriesController extends AppController
{
    /**
     * Index method
     *
     * @return \Cake\Http\Response|null|void Renders view
     */
    public function index()
    {
        $query = $this->ProductCategories->find()
            ->contain(['ProductCatalogs', 'ParentProductCategories']);
        $productCategories = $this->paginate($query);

        $this->set(compact('productCategories'));
    }

    /**
     * View method
     *
     * @param string|null $id Product Category id.
     * @return \Cake\Http\Response|null|void Renders view
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view(InternalCatalogManagerService $catalogManagerService, $id = null)
    {
        $productCategory = $catalogManagerService->getCategory($id);
        $this->set(compact('productCategory'));
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null|void Redirects on successful add, renders view otherwise.
     */
    public function add(InternalCatalogManagerService $catalogManagerService)
    {
        $productCategory = $this->ProductCategories->newEmptyEntity();
        if ($this->request->is('post')) {
            $postData = $this->request->getData();
            if ($this->request->getSession()->read('Auth.User.id')) {
                $postData['created_by'] = $this->request->getSession()->read('Auth.User.id');
            }
            $result = $catalogManagerService->createNewCategory($productCategory, $postData);
            Log::debug(print_r('$result from createNewCategory', true));
            Log::debug(print_r($result, true));
            if ($result['result']) {
                $this->Flash->success(__('The product category has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The product category could not be saved. Please, try again.'));
        }
        $productCatalogs = $this->ProductCategories->ProductCatalogs->find('list', limit: 200)->all();
        $parentProductCategories = $this->ProductCategories->ParentProductCategories->find('list', limit: 200)->all();
        $this->set(compact('productCategory', 'productCatalogs', 'parentProductCategories'));
    }

    /**
     * Edit method
     *
     * @param string|null $id Product Category id.
     * @return \Cake\Http\Response|null|void Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit(InternalCatalogManagerService $catalogManagerService, $id = null)
    {
        $productCategory = $this->ProductCategories->get($id, contain: []);
        if ($this->request->is(['patch', 'post', 'put'])) {
            $productCategory = $this->ProductCategories->patchEntity($productCategory, $this->request->getData());
            if ($this->ProductCategories->save($productCategory)) {
                $this->Flash->success(__('The product category has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The product category could not be saved. Please, try again.'));
        }
        $productCatalogs = $this->ProductCategories->ProductCatalogs->find('list', limit: 200)->all();
        $parentProductCategories = $this->ProductCategories->ParentProductCategories->find('list', limit: 200)->all();
        $this->set(compact('productCategory', 'productCatalogs', 'parentProductCategories'));
    }

    /**
     * Delete method
     *
     * @param string|null $id Product Category id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $productCategory = $this->ProductCategories->get($id);
        if ($this->ProductCategories->delete($productCategory)) {
            $this->Flash->success(__('The product category has been deleted.'));
        } else {
            $this->Flash->error(__('The product category could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }
}
