<?php

namespace CakeProducts\Service;

use Cake\Cache\Cache;
use Cake\Core\ServiceConfig;
use Cake\Http\Client;
use Cake\I18n\FrozenTime;
use Cake\I18n\Time;
use Cake\Log\Log;
use Cake\ORM\Locator\LocatorAwareTrait;
use CakeProducts\Model\Entity\ExternalProductCatalog;
use CakeProducts\Model\Entity\ProductCategory;
use CakeProducts\Model\Table\ProductCatalogsTable;

class ExternalCatalogManagerService
{
    use LocatorAwareTrait;

    /**
     * @var ProductCatalogsTable
     */
    protected \Cake\ORM\Table|ProductCatalogsTable $ProductCatalogs;

    /**
     * @var ServiceConfig
     */
    protected ServiceConfig $serviceConfig;

    /**
     * @var Client
     */
    protected Client $httpClient;

    /**
     *
     */
    public function __construct()
    {
        $this->ProductCatalogs = $this->fetchTable('CakeProducts.ProductCatalogs');
        $this->serviceConfig = new ServiceConfig();
        $this->httpClient = new Client([
//            'host' => $config['base_url'],
//            'scheme' => 'https',
//            'scheme' => 'http',
        ]);

    }

    public function newCategoryCreated(ProductCategory $productCategory)
    {
        $results = [];

        $externalProductCatalogs = $this->_getExternalProductCatalogsForCatalogId($productCategory->product_catalog_id);
        foreach ($externalProductCatalogs as $externalProductCatalog) {
            $results[] = $this->_createNewCategoryForExternalProductCatalog($externalProductCatalog, $productCategory);
        }

        return $results;
    }

    protected function _createNewCategoryForExternalProductCatalog(ExternalProductCatalog $externalProductCatalog, ProductCategory $productCategory)
    {
        $url = $externalProductCatalog->api_url . '/product-categories';
        $response = $this->postToUrl($url, $productCategory->toArray());

        Log::debug(print_r('$response->getJson()', true));
        Log::debug(print_r($response->getJson(), true));
        Log::debug(print_r('$response->getStatusCode()', true));
        Log::debug(print_r($response->getStatusCode(), true));

        return $response->getStatusCode();
    }

    /**
     * @return mixed|null
     */
    public function getJwtToken()
    {
        Log::debug('inside getJwtToken');
        if (Cache::read('product_catalog_api_token')) {
            Log::debug('token was cached');
//            return Cache::read('product_catalog_api_token');
        } else {
            Log::debug('token was **NOT** cached');
        }

        $response = $this->httpClient->post('http://localhost:8766/api/v1/users/token', json_encode([
            'username' => 'test',
            'password' => 'test',
        ]), ['headers' => ['Accept' => 'application/json', 'Content-Type' => 'application/json']]);
//        $this->httpClient->getConfig();
        if ($response->isOk()) {
            $json = $response->getJson();
            $token = array_key_exists('token', $json) ? $json['token'] : null;
            Cache::write('product_catalog_api_token', $token);
            Log::debug('$token');
            Log::debug($token);

            return $token;
        }
        Log::debug('$response->getStringBody()');
        Log::debug($response->getStringBody());
        Log::debug(print_r('$response->getStatusCode()', true));
        Log::debug(print_r($response->getStatusCode(), true));

        return null;
    }

    public function postToUrl(string $url, array $data, int $tries = 0)
    {
//        if (true || !Cache::read('product_catalog_api_token')) {
        $token = $this->getJwtToken();
//        }
        Log::debug('$token inside postToUrl' . $token);

        Log::debug('Cache::read(product_catalog_api_token)');
        Log::debug(Cache::read('product_catalog_api_token') ? Cache::read('product_catalog_api_token') : 'NULL');
        Log::debug('ATTEMPT # ' . $tries);

        $response = $this->httpClient->post($url, json_encode($data), [
            'headers' => [
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
//                'Authorization' => 'Bearer ' . base64_encode(Cache::read('product_catalog_api_token'))
                'Authorization' => 'Bearer ' . $token,
            ]
        ]);

        if (!$response->isOk()) {
            $tries++;
        }
        if ($tries > 3) {
            return $response;
        }
        if ($response->getStatusCode() == 401) {
            $this->postToUrl($url, $data, $tries);
        }
        Log::debug('$response->getJson');
        Log::debug(print_r($response->getJson(), true));

        return $response;
    }

    /**
     * @param string $url
     * @param array $data
     * @param int $tries
     *
     * @return mixed
     */
    public function putToUrl(string $url, array $data, int $tries = 0)
    {
        if (!Cache::read('product_catalog_api_token')) {
            $this->getJwtToken();
        }

        $response = $this->httpClient->put($url, json_encode($data), [
            'headers' => [
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . Cache::read('product_catalog_api_token')
            ],
        ]);

        if (!$response->isOk()) {
            $tries++;
        }
        if ($tries > 3) {
            return $response;
        }
        if ($response->getStatusCode() == 401) {
            $this->getJwtToken();
            $this->putToUrl($url, $data, $tries);
        }

        return $response;
    }

    protected function _getExternalProductCatalogsForCatalogId(string $productCatalogId)
    {
        return $this->ProductCatalogs->ExternalProductCatalogs->find()->where(['product_catalog_id' => $productCatalogId])->toArray();
    }
}
